// ESP8266Transceiver.cpp
//
// Class for WiShield based RCOIP transceiver
/// \author  Mike McCauley (mikem@airspayce.com)
///
// Copyright (C) 2012 Mike McCauley
// $Id: ESP8266Transceiver.cpp,v 1.1 2018/09/17 23:09:57 mikem Exp mikem $

#ifdef ESP8266
#include "ESP8266Transceiver.h"
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include "RCRx.h"

// Remember the singleton instance of the ESP8266Transceiver
// Need this because of ugly C glue code
ESP8266Transceiver* theTransceiver = NULL;

// Interface and configuration of the ESP8266 built in WiFi
// Change these if you want
WiFiUDP Udp;
unsigned int localUdpPort = RCOIP_DEFAULT_RECEIVER_UDP_PORT;
char*     ssid = "RCArduino";      // Access Point SSID
char*     password = "xyzzyxyzzy"; // Must be at least 8 chars
int       channel = 1;             // WiFi channel
IPAddress local_IP(192,168,4,1);
IPAddress gateway(192,168,4,1);
IPAddress subnet(255,255,255,0);

/////////////////////////////////////////////////////////////////////
ESP8266Transceiver::ESP8266Transceiver()
{
    theTransceiver = this;
}

/////////////////////////////////////////////////////////////////////
void ESP8266Transceiver::init()
{
    // Set up the soft AP
    // We become an access point that the WiFi sender can connect to
    boolean result;
    result = WiFi.softAPConfig(local_IP, gateway, subnet);
    if (!result)
	Serial.println("ESP8266Transceiver::init WiFi.softAPConfig failed");
	
    result = WiFi.softAP(ssid, password, channel);
    if (!result)
	Serial.println("ESP8266Transceiver::init WiFi.softAP failed");

//    Serial.print("Soft-AP IP address = ");
//    Serial.println(WiFi.softAPIP());
    Udp.begin(localUdpPort);
}

/////////////////////////////////////////////////////////////////////
void ESP8266Transceiver::run()
{
    int packetSize = Udp.parsePacket();
    if (packetSize)
    {
	// Get the incoming UDP packet
//	Serial.printf("Received %d bytes from %s, port %d, RSSI %d\n", packetSize, Udp.remoteIP().toString().c_str(), Udp.remotePort(), WiFi.RSSI());
	if (theTransceiver)
	{
	    // And give it to RCRx to decode and distribute
	    char incomingPacket[255];
	    int len = Udp.read(incomingPacket, sizeof(incomingPacket));
	    theTransceiver->receivedRequest((uint8_t*)incomingPacket, len, WiFi.RSSI() + 60);
	}
    }
}

/////////////////////////////////////////////////////////////////////
void ESP8266Transceiver::sendReply(uint8_t* msg, uint16_t len)
{
    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
    Udp.write(msg, len);
    Udp.endPacket();
}

#endif
