// ESP8266Transceiver.h
//
// RCRx transceiver device for ESP8266 devices
// Copyright (C) 2018 Mike McCauley
// $Id: ESP8266Transceiver.h,v 1.1 2018/09/17 23:09:57 mikem Exp mikem $

#ifndef ESP8266Transceiver_h
#define ESP8266Transceiver_h

#include "Transceiver.h"
#include "RCOIP.h"

/////////////////////////////////////////////////////////////////////
/// \class ESP8266Transceiver ESP8266Transceiver.h <ESP8266Transceiver.h>
/// \brief Defines a WiSHield RCOIP Transceiver
///
/// This is one of several types of Transceiver that RCRx can use to communicate with
/// an RCOIP transmitter.
/// It works with the WiFi device on an ESP8266 based device
///
/// The default WiFi configuration is:
/// - IP Address: 169.254.1.100
/// - Gateway and transmitter address: 169.254.1.1
/// - SSID: RCArduino
/// - WiFi security type: NONE (i.e. an open unencrypted, no password WiFi network)
/// - WiFi mode: ad-hoc (ie direct node-to-node communications)
/// You can change these default settings by editing ESP8266Transceiver.cpp. (I know thats ugly, but
/// it is a consequence of the way the WiShield library is built).
///
/// There is only permitted to be one instance of ESP8266Transceiver. This is because of the low level 
/// C code that glues the WiShield code to ESP8266Transceiver, and the fact there can also be only one instance of 
/// the WiShield object.
class ESP8266Transceiver : public Transceiver
{   
public:
    /// Constructor.
    /// Creates a new ESP8266Transceiver object.
    ESP8266Transceiver();

    /// Initialise the WiShield using the WiFi configuration in ESP8266Transceiver.cpp
    /// Call this once before using the Transceiver
    virtual void init();

    /// Poll the object for activity.
    /// This is expected to be called frequently in the main loop.
    /// It processes the WiFi stack, checking for received messages.
    /// During processing, and RCOIP message received by the preconfigured port and address
    /// will be given to the RCRx object pointed to by _delegate
    virtual void run();

    /// Send an RCOIP reply message to the sender of the current received message.
    /// \param[in] msg Pointer to the RCOP message.
    /// \param[in] len Length of the RCOIP message in bytes.
    virtual void   sendReply(uint8_t* msg, uint16_t len);
};

#endif 
